require "octokit"

class Todoplicator
  class GitHubAdapter
    class Issue
      def initialize(raw)
        @raw = raw
      end

      def title
        @raw.title
      end

      def body
        @raw.body
      end

      def state
        @raw.state
      end

      def raw
        @raw
      end

      def closed?
        state == "closed"
      end

      def open?
        state == "open"
      end
    end

    AdapterRegistry.instance.add(:github, GitHubAdapter)

    def initialize(target)
      @target = target
    end

    def find_issues
      client.list_issues(@target, state: :all).map { |i| Issue.new(i) }
    end

    def create_issue(issue)
      new_issue = Issue.new(client.create_issue(@target, issue.title, issue.body))
      new_issue = Issue.new(client.close_issue(@target, new_issue.raw.number)) if issue.closed?
      new_issue
    end

  private

    def client
      @client ||= Octokit::Client.new(
        access_token: ENV["GITHUB_TOKEN"].to_s
      )
    end
  end
end
