require "gitlab"

class Todoplicator
  class GitLabAdapter
    class Issue
      def initialize(raw)
        @raw = raw
      end

      def title
        @raw.title
      end

      def body
        @raw.description
      end

      def state
        @raw.state
      end

      def raw
        @raw
      end

      def closed?
        state == "closed"
      end

      def open?
        state == "opened"
      end
    end

    AdapterRegistry.instance.add(:gitlab, GitLabAdapter)

    def initialize(target)
      @target = target
    end

    def find_issues
      client.issues(@target).map { |i| Issue.new(i) }
    end

    def create_issue(issue)
      new_issue = Issue.new(client.create_issue(@target, issue.title, description: issue.body))
      new_issue = Issue.new(client.close_issue(@target, new_issue.raw.iid)) if issue.closed?
      new_issue
    end

  private

    def client
      @client ||= Gitlab.client(
        endpoint: "https://gitlab.com/api/v4",
        private_token: ENV["GITLAB_TOKEN"].to_s
      )
    end
  end
end
